(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var _ = Package.underscore._;
var AppConfig = Package['application-configuration'].AppConfig;

/* Package-scope variables */
var SandstormEmail;

(function () {

////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
// packages/sandstorm-email/email.js                                                          //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                              //
var Future = Npm.require('fibers/future');                                                    // 1
var urlModule = Npm.require('url');                                                           // 2
var MailComposer = Npm.require('mailcomposer').MailComposer;                                  // 3
                                                                                              // 4
SandstormEmail = {};                                                                          // 5
                                                                                              // 6
var getSmtpUrl = function () {                                                                // 7
  var setting = Settings.findOne({_id: "smtpUrl"});                                           // 8
  if (setting) {                                                                              // 9
    return setting.value;                                                                     // 10
  } else {                                                                                    // 11
    return process.env.MAIL_URL;                                                              // 12
  }                                                                                           // 13
};                                                                                            // 14
                                                                                              // 15
var makePool = function (mailUrlString) {                                                     // 16
  var mailUrl = urlModule.parse(mailUrlString);                                               // 17
  if (mailUrl.protocol !== 'smtp:')                                                           // 18
    throw new Error("Email protocol in $MAIL_URL (" +                                         // 19
                    mailUrlString + ") must be 'smtp'");                                      // 20
                                                                                              // 21
  var port = +(mailUrl.port);                                                                 // 22
  var auth = false;                                                                           // 23
  if (mailUrl.auth) {                                                                         // 24
    var parts = mailUrl.auth.split(':', 2);                                                   // 25
    auth = {user: parts[0] && decodeURIComponent(parts[0]),                                   // 26
            pass: parts[1] && decodeURIComponent(parts[1])};                                  // 27
  }                                                                                           // 28
                                                                                              // 29
  var simplesmtp = Npm.require('simplesmtp');                                                 // 30
  var pool = simplesmtp.createClientPool(                                                     // 31
    port,  // Defaults to 25                                                                  // 32
    mailUrl.hostname,  // Defaults to "localhost"                                             // 33
    { secureConnection: (port === 465),                                                       // 34
      // XXX allow maxConnections to be configured?                                           // 35
      auth: auth });                                                                          // 36
                                                                                              // 37
  pool._future_wrapped_sendMail = _.bind(Future.wrap(pool.sendMail), pool);                   // 38
  return pool;                                                                                // 39
};                                                                                            // 40
                                                                                              // 41
// We construct smtpPool at the first call to Email.send, so that                             // 42
// Meteor.startup code can set $MAIL_URL.                                                     // 43
var pool;                                                                                     // 44
var configured = false;                                                                       // 45
                                                                                              // 46
Meteor.startup(function() {                                                                   // 47
  Settings.find({_id: "smtpUrl"}).observeChanges({                                            // 48
    removed : function () {                                                                   // 49
      configured = false;                                                                     // 50
    },                                                                                        // 51
    changed : function () {                                                                   // 52
      configured = false;                                                                     // 53
    },                                                                                        // 54
    added : function () {                                                                     // 55
      configured = false;                                                                     // 56
    }                                                                                         // 57
  });                                                                                         // 58
                                                                                              // 59
  // Accounts.emailToken is set to use "Email" by default. Change it to use our mail service. // 60
  Accounts.emailToken.setEmailPackage("SandstormEmail");                                      // 61
});                                                                                           // 62
                                                                                              // 63
var getPool = function () {                                                                   // 64
  // We check MAIL_URL in case someone else set it in Meteor.startup code.                    // 65
  if (!configured) {                                                                          // 66
    configured = true;                                                                        // 67
    var url = getSmtpUrl();                                                                   // 68
    if (url)                                                                                  // 69
      pool = makePool(url);                                                                   // 70
  }                                                                                           // 71
                                                                                              // 72
  return pool;                                                                                // 73
};                                                                                            // 74
                                                                                              // 75
var next_devmode_mail_id = 0;                                                                 // 76
var output_stream = process.stdout;                                                           // 77
                                                                                              // 78
var devModeSend = function (mc) {                                                             // 79
  var devmode_mail_id = next_devmode_mail_id++;                                               // 80
                                                                                              // 81
  var stream = output_stream;                                                                 // 82
                                                                                              // 83
  // This approach does not prevent other writers to stdout from interleaving.                // 84
  stream.write("====== BEGIN MAIL #" + devmode_mail_id + " ======\n");                        // 85
  stream.write("(Mail not sent; to enable sending, set the MAIL_URL " +                       // 86
               "environment variable.)\n");                                                   // 87
  mc.streamMessage();                                                                         // 88
  mc.pipe(stream, {end: false});                                                              // 89
  var future = new Future;                                                                    // 90
  mc.on('end', function () {                                                                  // 91
    stream.write("====== END MAIL #" + devmode_mail_id + " ======\n");                        // 92
    future['return']();                                                                       // 93
  });                                                                                         // 94
  future.wait();                                                                              // 95
};                                                                                            // 96
                                                                                              // 97
var smtpSend = function (pool, mc) {                                                          // 98
  pool._future_wrapped_sendMail(mc).wait();                                                   // 99
};                                                                                            // 100
                                                                                              // 101
// Old comment below                                                                          // 102
/**                                                                                           // 103
 * Send an email.                                                                             // 104
 *                                                                                            // 105
 * Connects to the mail server configured via the MAIL_URL environment                        // 106
 * variable. If unset, prints formatted message to stdout. The "from" option                  // 107
 * is required, and at least one of "to", "cc", and "bcc" must be provided;                   // 108
 * all other options are optional.                                                            // 109
 *                                                                                            // 110
 * @param options                                                                             // 111
 * @param options.from {String} RFC5322 "From:" address                                       // 112
 * @param options.to {String|String[]} RFC5322 "To:" address[es]                              // 113
 * @param options.cc {String|String[]} RFC5322 "Cc:" address[es]                              // 114
 * @param options.bcc {String|String[]} RFC5322 "Bcc:" address[es]                            // 115
 * @param options.replyTo {String|String[]} RFC5322 "Reply-To:" address[es]                   // 116
 * @param options.subject {String} RFC5322 "Subject:" line                                    // 117
 * @param options.text {String} RFC5322 mail body (plain text)                                // 118
 * @param options.html {String} RFC5322 mail body (HTML)                                      // 119
 * @param options.headers {Object} custom RFC5322 headers (dictionary)                        // 120
 */                                                                                           // 121
                                                                                              // 122
// New API doc comment below                                                                  // 123
/**                                                                                           // 124
 * @summary Send an email. Throws an `Error` on failure to contact mail server                // 125
 * or if mail server returns an error. All fields should match                                // 126
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.                               // 127
 * @locus Server                                                                              // 128
 * @param {Object} options                                                                    // 129
 * @param {String} options.from "From:" address (required)                                    // 130
 * @param {String|String[]} options.to,cc,bcc,replyTo                                         // 131
 *   "To:", "Cc:", "Bcc:", and "Reply-To:" addresses                                          // 132
 * @param {String} [options.subject]  "Subject:" line                                         // 133
 * @param {String} [options.text|html] Mail body (in plain text or HTML)                      // 134
 * @param {Object} [options.headers] Dictionary of custom headers                             // 135
 */                                                                                           // 136
SandstormEmail.send = function (options) {                                                    // 137
  var mc = new MailComposer();                                                                // 138
                                                                                              // 139
  // setup message data                                                                       // 140
  // XXX support attachments (once we have a client/server-compatible binary                  // 141
  //     Buffer class)                                                                        // 142
  mc.setMessageOption({                                                                       // 143
    from: options.from,                                                                       // 144
    to: options.to,                                                                           // 145
    cc: options.cc,                                                                           // 146
    bcc: options.bcc,                                                                         // 147
    replyTo: options.replyTo,                                                                 // 148
    subject: options.subject,                                                                 // 149
    text: options.text,                                                                       // 150
    html: options.html                                                                        // 151
  });                                                                                         // 152
                                                                                              // 153
  _.each(options.headers, function (value, name) {                                            // 154
    mc.addHeader(name, value);                                                                // 155
  });                                                                                         // 156
                                                                                              // 157
  SandstormEmail.rawSend(mc);                                                                 // 158
};                                                                                            // 159
                                                                                              // 160
/**                                                                                           // 161
 * @summary Sends a raw email with a MailComposer object.                                     // 162
 * Throws an `Error` on failure to contact mail server                                        // 163
 * or if mail server returns an error. All fields should match                                // 164
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.                               // 165
 * @locus Server                                                                              // 166
 * @param {Object} mc A MailCompser object that you wish to send                              // 167
*/                                                                                            // 168
SandstormEmail.rawSend = function (mc) {                                                      // 169
  var pool = getPool();                                                                       // 170
  if (pool) {                                                                                 // 171
    smtpSend(pool, mc);                                                                       // 172
  } else {                                                                                    // 173
    throw new Exception("SMTP pool is misconfigured.");                                       // 174
  }                                                                                           // 175
};                                                                                            // 176
                                                                                              // 177
////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-email'] = {
  SandstormEmail: SandstormEmail
};

})();

//# sourceMappingURL=sandstorm-email.js.map
